import React, { useEffect, useState } from 'react';
import Uuid from 'uuid';
import noop from 'lodash/noop';
import classnames from 'classnames';
import PropTypes from 'prop-types';
import './dialog.less';
import Icon from './../icon/icon';

export default function Dialog({
    contentChildren,
    contentHeight,
    footerChildren,
    headerChildren,
    id,
    isModal,
    isOpen,
    isClosable,
    showFooter,
    showHeader,
    subClass,
    width,
    contentPadding,
    closeOnEscape,
    closeOnEnter,
    onEnter,
    onEscape,
    onClose
}) {
    const [isOpenState, setIsOpenState] = useState(isOpen);
    const className = classnames(
        'tsdev-dialog', subClass
    );
    const inlineStyleWrapper = {
        width: width,
        display: isOpenState ? 'block' : 'none'
    };
    const inlineStyleHeader = {
        display: showHeader ? 'flex' : 'none'
    };
    const inlineStyleFooter = {
        display: showFooter ? 'block' : 'none'
    };
    const inlineStyleContent = {
        padding: contentPadding,
        width: width - 4 * contentPadding,
        height: contentHeight
    };
    const inlineStyleOverlay = {
        display: isOpenState && isModal ? 'block' : 'none'
    };
    const inlineStyleCloseIcon = {
        display: isClosable ? 'block' : 'none'
    };
    const handleCloseDialog = () => /* istanbul ignore next */ {
        if (isClosable) {
            setIsOpenState(false);
            onClose({
                id
            });
        }
    };

    /* istanbul ignore next */
    const handleKeyDown = (event) => {
        if (!isClosable) {
            return;
        }
        if (closeOnEscape && (event.key === 'Escape' || event.key === 'Esc')) {
            handleCloseDialog();
            onEscape({
                id
            });
            event.stopPropagation();
            event.preventDefault();
        }
        if (event.key === 'Enter') {
            if (closeOnEnter) {
                handleCloseDialog();
            }
            onEnter({
                id
            });
            event.stopPropagation();
            event.preventDefault();
        }
    };

    useEffect(() => {
        window.addEventListener('keydown', handleKeyDown);
        setIsOpenState(isOpen);

        /* istanbul ignore next */
        return () => {
            window.removeEventListener('keydown', handleKeyDown);
        };
    }, [isOpen, isClosable, closeOnEnter, closeOnEscape]);

    return (
        <div id={id}
            className={className}>
            <div className="tsdev-dialog-wrapper"
                style={inlineStyleWrapper}>
                <div className="tsdev-dialog-header"
                    style={inlineStyleHeader}>
                    {headerChildren}
                    <div className="close-icon"
                        onClick={handleCloseDialog}
                        style={inlineStyleCloseIcon}>
                        <Icon iconType="close"
                            size={20} />
                    </div>
                </div>
                <div className="tsdev-dialog-content"
                    style={inlineStyleContent}>
                    {contentChildren}
                </div>
                <div className="tsdev-dialog-footer"
                    style={inlineStyleFooter}>
                    {footerChildren}
                </div>
            </div>
            <div className="tsdev-dialog-overlay"
                style={inlineStyleOverlay} />
        </div >
    );
}

Dialog.propTypes = {
    closeOnEnter: PropTypes.bool,
    closeOnEscape: PropTypes.bool,
    contentChildren: PropTypes.node,
    contentHeight: PropTypes.oneOfType([
        PropTypes.string,
        PropTypes.number
    ]),
    contentPadding: PropTypes.number,
    footerChildren: PropTypes.node,
    headerChildren: PropTypes.node,
    id: PropTypes.string,
    isClosable: PropTypes.bool,
    isModal: PropTypes.bool,
    isOpen: PropTypes.bool,
    onClose: PropTypes.func,
    onEnter: PropTypes.func,
    onEscape: PropTypes.func,
    showFooter: PropTypes.bool,
    showHeader: PropTypes.bool,
    subClass: PropTypes.string,
    width: PropTypes.oneOfType([
        PropTypes.string,
        PropTypes.number
    ])
};

Dialog.defaultProps = {
    closeOnEnter: true,
    closeOnEscape: true,
    contentChildren: (<div>Content</div>),
    contentHeight: 200,
    contentPadding: 8,
    footerChildren: (<div>Footer</div>),
    headerChildren: (<div>Header</div>),
    id: `tsdevDialog-${Uuid.v4()}`,
    isClosable: true,
    isModal: false,
    isOpen: false,
    onClose: noop,
    onEnter: noop,
    onEscape: noop,
    showFooter: true,
    showHeader: true,
    subClass: '',
    width: 450
};

Dialog.displayName = 'Dialog';
