import React, { Component } from 'react';
import map from 'lodash/map';
import each from 'lodash/each';
import noop from 'lodash/noop';
import classnames from 'classnames';
import PropTypes from 'prop-types';
import Uuid from 'uuid';
import './tag-select.less';
import Tag from './../tag/tag';
import Icon from '../icon/icon';

export default class TagSelect extends Component {

    constructor(props) {
        super(props);
        this.state = {
            expanded: props.expanded,
            availableItems: this.props.availableItems
        };
        this.handleOnCollapse = this.handleOnCollapse.bind(this);
        this.handleOnRemove = this.handleOnRemove.bind(this);
    }

    /* istanbul ignore next */
    static getDerivedStateFromProps(props, state) {
        return {
            expanded: state.expanded
        };
    }

    handleOnItemClick(event) {
        const {
            availableItems
        } = this.state;
        const {
            onSelect,
            closeOnSelect
        } = this.props;
        const id = Number(event.target.getAttribute('data-attribute-id'));
        const disabled = event.target.getAttribute('data-attribute-is-disabled') === 'true';

        event.stopPropagation();
        /* istanbul ignore else */
        if (!disabled) {
            each(availableItems, (availableItem) => {
                if (availableItem.id === id) {
                    availableItem.selected = availableItem.selected ? /* istanbul ignore next */ false : true;
                }
            });
            this.setState({
                availableItems: availableItems
            });
            if (closeOnSelect) {
                this.setState({
                    expanded: false
                });
            }
            onSelect(event, {
                id: Number(event.target.getAttribute('data-attribute-id')),
                label: event.target.getAttribute('data-attribute-label'),
                value: Number(event.target.getAttribute('data-attribute-value')),
                selected: event.target.getAttribute('data-attribute-is-selected') === 'true',
                disabled: event.target.getAttribute('data-attribute-is-disabled') === 'true'
            }, availableItems);
        }
    }

    handleOnCollapse(event) {
        const {
            expanded
        } = this.state;
        const {
            disabled
        } = this.props;

        event.stopPropagation();

        /* istanbul ignore else */
        if (!disabled) {
            this.setState({
                expanded: !expanded
            });
        }
    }

    /* istanbul ignore next */
    handleOnRemove(item) {
        const {
            availableItems
        } = this.state;
        const {
            onCloseTag,
            disabled
        } = this.props;

        if (!disabled) {
            each(availableItems, (availableItem) => {
                if (availableItem.label === item.label) {
                    availableItem.selected = false;
                }
            });
            this.setState({
                availableItems: availableItems
            });
            onCloseTag(event, item);
        }
    }

    render() {
        const {
            id,
            subClass,
            placeholder,
            summaryHeight,
            showSummary,
            width,
            disabled,
            isTagClosable,
            showValue,
            showSelect
        } = this.props;
        const {
            expanded,
            availableItems,
            currentItem
        } = this.state;
        const className = classnames(
            'tsdev-tag-select', subClass
        );
        const classNameDropDown = classnames('tsdev-tag-select-drop-down', disabled ? 'disabled' : '');
        const classNameSummary = classnames('tsdev-tag-select-summary', disabled ? 'disabled' : '');
        const inlineStyleSelectionList = {
            display: expanded ? 'block' : 'none'
        };
        const inlineStyle = {
            width: width
        };
        const inlineStyleSummary = {
            display: showSummary ? 'block' : 'none',
            height: summaryHeight
        };
        const inlineStyleDropDown = {
            display: showSelect ? 'block' : 'none',
            paddingBottom: expanded ? 15 : 4
        };
        const availableItemsToRender = map(availableItems, (availableItem, index) => {
            const {
                showDisabledItems
            } = this.props;

            if (!availableItem.selected || (!showDisabledItems && /* istanbul ignore next */ availableItem.disabled)) {
                return null;
            }

            return (
                <Tag key={`selected-item-${index}`}
                    id={`selected-item-${id}-${index}`}
                    isClosable={isTagClosable}
                    showValue={showValue}
                    disabled={availableItem.disabled || disabled}
                    value={availableItem.value}
                    onClose={this.handleOnRemove}
                    label={availableItem.label} />
            );
        });
        const itemsToRender = map(availableItems, (availableItem, index) => {
            const {
                showDisabledItems
            } = this.props;
            const classNameAvailable = classnames(
                'available-item',
                availableItem.selected ? 'selected' : '',
                availableItem.disabled ? 'disabled' : ''
            );

            /* istanbul ignore next */
            if (!showDisabledItems && availableItem.disabled) {
                return null;
            }

            return (
                <div key={`available-item-${index}`}
                    onClick={(event) => this.handleOnItemClick(event, currentItem)}
                    className={classNameAvailable}
                    data-attribute-id={availableItem.id}
                    data-attribute-label={availableItem.label}
                    data-attribute-value={availableItem.value}
                    data-attribute-is-selected={availableItem.selected}
                    data-attribute-is-disabled={availableItem.disabled}
                    id={`available-item-${id}-${index}`}>
                    {availableItem.label}
                </div>
            );
        });

        return (
            <div id={id}
                style={inlineStyle}
                className={className}>
                <div className={classNameSummary}
                    style={inlineStyleSummary}>
                    {availableItemsToRender}
                </div>
                <div className={classNameDropDown}
                    style={inlineStyleDropDown}
                    onClick={this.handleOnCollapse}>
                    <div className="tsdev-tag-select-drop-down-icon">
                        <Icon size={14}
                            iconType={expanded ? 'arrow-down' : 'arrow-right'} />
                    </div>
                    <div className="tsdev-tag-select-drop-down-placeholder">
                        {placeholder}
                    </div>
                    <div className="tsdev-tag-select-drop-down-selection-list"
                        style={inlineStyleSelectionList}>
                        {itemsToRender}
                    </div>
                </div>
            </div>
        );
    }
}

TagSelect.propTypes = {
    availableItems: PropTypes.arrayOf(PropTypes.shape({
        id: PropTypes.number,
        label: PropTypes.string,
        value: PropTypes.number,
        selected: PropTypes.bool,
        disabled: PropTypes.bool
    })),
    closeOnSelect: PropTypes.bool,
    disabled: PropTypes.bool,
    expanded: PropTypes.bool,
    id: PropTypes.string,
    isTagClosable: PropTypes.bool,
    onCloseTag: PropTypes.func,
    onSelect: PropTypes.func,
    placeholder: PropTypes.string,
    showSelect: PropTypes.bool,
    showDisabledItems: PropTypes.bool,
    showSummary: PropTypes.bool,
    showValue: PropTypes.bool,
    subClass: PropTypes.string,
    summaryHeight: PropTypes.number,
    width: PropTypes.oneOfType([
        PropTypes.string,
        PropTypes.number
    ])
};

TagSelect.defaultProps = {
    availableItems: [
        {
            id: 1,
            label: 'Item-1',
            value: 10,
            selected: false,
            disabled: false
        },
        {
            id: 2,
            label: 'Item-2',
            value: 20,
            selected: false,
            disabled: false
        },
        {
            id: 3,
            label: 'Item-3',
            value: 30,
            selected: true,
            disabled: false
        },
        {
            id: 4,
            label: 'Item-4',
            value: 40,
            selected: false,
            disabled: false
        },
        {
            id: 5,
            label: 'Item-5',
            value: 50,
            selected: true,
            disabled: true
        }
    ],
    closeOnSelect: false,
    disabled: false,
    expanded: false,
    id: `tsdevTag-${Uuid.v4()}`,
    isTagClosable: true,
    onCloseTag: noop,
    onSelect: noop,
    placeholder: 'Please Select',
    showDisabledItems: true,
    showSelect: true,
    showSummary: true,
    showValue: false,
    subClass: '',
    summaryHeight: 200,
    width: 'auto'
};

TagSelect.displayName = 'TagSelect';
